# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, strip_not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.bundle_pricing import billing_bundle_list_params
from ...types.bundle_pricing.billing_bundle_summary import BillingBundleSummary
from ...types.bundle_pricing.billing_bundle_retrieve_response import BillingBundleRetrieveResponse

__all__ = ["BillingBundlesResource", "AsyncBillingBundlesResource"]


class BillingBundlesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BillingBundlesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BillingBundlesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BillingBundlesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BillingBundlesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingBundleRetrieveResponse:
        """
        Get a single bundle by ID.

        Args:
          bundle_id: Billing bundle's ID, this is used to identify the billing bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bundle_id:
            raise ValueError(f"Expected a non-empty value for `bundle_id` but received {bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get(
            f"/bundle_pricing/billing_bundles/{bundle_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillingBundleRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: billing_bundle_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[BillingBundleSummary]:
        """
        Get all allowed bundles.

        Args:
          filter: Consolidated filter parameter (deepObject style). Supports filtering by
              country_iso and resource. Examples: filter[country_iso]=US or
              filter[resource]=+15617819942

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get_api_list(
            "/bundle_pricing/billing_bundles",
            page=SyncDefaultFlatPagination[BillingBundleSummary],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    billing_bundle_list_params.BillingBundleListParams,
                ),
            ),
            model=BillingBundleSummary,
        )


class AsyncBillingBundlesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBillingBundlesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBillingBundlesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBillingBundlesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBillingBundlesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingBundleRetrieveResponse:
        """
        Get a single bundle by ID.

        Args:
          bundle_id: Billing bundle's ID, this is used to identify the billing bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bundle_id:
            raise ValueError(f"Expected a non-empty value for `bundle_id` but received {bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return await self._get(
            f"/bundle_pricing/billing_bundles/{bundle_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillingBundleRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: billing_bundle_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BillingBundleSummary, AsyncDefaultFlatPagination[BillingBundleSummary]]:
        """
        Get all allowed bundles.

        Args:
          filter: Consolidated filter parameter (deepObject style). Supports filtering by
              country_iso and resource. Examples: filter[country_iso]=US or
              filter[resource]=+15617819942

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get_api_list(
            "/bundle_pricing/billing_bundles",
            page=AsyncDefaultFlatPagination[BillingBundleSummary],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    billing_bundle_list_params.BillingBundleListParams,
                ),
            ),
            model=BillingBundleSummary,
        )


class BillingBundlesResourceWithRawResponse:
    def __init__(self, billing_bundles: BillingBundlesResource) -> None:
        self._billing_bundles = billing_bundles

        self.retrieve = to_raw_response_wrapper(
            billing_bundles.retrieve,
        )
        self.list = to_raw_response_wrapper(
            billing_bundles.list,
        )


class AsyncBillingBundlesResourceWithRawResponse:
    def __init__(self, billing_bundles: AsyncBillingBundlesResource) -> None:
        self._billing_bundles = billing_bundles

        self.retrieve = async_to_raw_response_wrapper(
            billing_bundles.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            billing_bundles.list,
        )


class BillingBundlesResourceWithStreamingResponse:
    def __init__(self, billing_bundles: BillingBundlesResource) -> None:
        self._billing_bundles = billing_bundles

        self.retrieve = to_streamed_response_wrapper(
            billing_bundles.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            billing_bundles.list,
        )


class AsyncBillingBundlesResourceWithStreamingResponse:
    def __init__(self, billing_bundles: AsyncBillingBundlesResource) -> None:
        self._billing_bundles = billing_bundles

        self.retrieve = async_to_streamed_response_wrapper(
            billing_bundles.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            billing_bundles.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ActionShareResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this sharing token"""

    token: Optional[str] = None
    """A signed JWT token that can be used to access the shared resource"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    expires_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the sharing token expires."""

    expires_in_seconds: Optional[int] = None
    """The number of seconds until the sharing token expires"""

    permissions: Optional[List[Literal["porting_order.document.read", "porting_order.document.update"]]] = None
    """The permissions granted to the sharing token"""

    porting_order_id: Optional[str] = None
    """Identifies the porting order resource being shared"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class ActionShareResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .record import Record
from .._models import BaseModel
from .network_interface import NetworkInterface

__all__ = ["PublicInternetGatewayCreateResponse", "Data"]


class Data(Record, NetworkInterface):
    public_ip: Optional[str] = None
    """The publically accessible ip for this interface."""

    region_code: Optional[str] = None
    """The region interface is deployed to."""


class PublicInternetGatewayCreateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ParticipantUpdateResponse"]


class ParticipantUpdateResponse(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    api_version: Optional[str] = None
    """The version of the API that was used to make the request."""

    call_sid: Optional[str] = None
    """The identifier of this participant's call."""

    call_sid_legacy: Optional[str] = None
    """The identifier of this participant's call."""

    coaching: Optional[bool] = None
    """Whether the participant is coaching another call."""

    coaching_call_sid: Optional[str] = None
    """The identifier of the coached participant's call."""

    coaching_call_sid_legacy: Optional[str] = None
    """The identifier of the coached participant's call."""

    date_created: Optional[str] = None
    """The timestamp of when the resource was created."""

    date_updated: Optional[str] = None
    """The timestamp of when the resource was last updated."""

    end_conference_on_exit: Optional[bool] = None
    """Whether the conference ends when the participant leaves."""

    hold: Optional[bool] = None
    """Whether the participant is on hold."""

    muted: Optional[bool] = None
    """Whether the participant is muted."""

    status: Optional[Literal["connecting", "connected", "completed"]] = None
    """The status of the participant's call in the conference."""

    uri: Optional[str] = None
    """The relative URI for this participant."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import virtual_cross_connects_coverage_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.virtual_cross_connects_coverage_list_response import VirtualCrossConnectsCoverageListResponse

__all__ = ["VirtualCrossConnectsCoverageResource", "AsyncVirtualCrossConnectsCoverageResource"]


class VirtualCrossConnectsCoverageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VirtualCrossConnectsCoverageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VirtualCrossConnectsCoverageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VirtualCrossConnectsCoverageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VirtualCrossConnectsCoverageResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: virtual_cross_connects_coverage_list_params.Filter | Omit = omit,
        filters: virtual_cross_connects_coverage_list_params.Filters | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[VirtualCrossConnectsCoverageListResponse]:
        """
        List Virtual Cross Connects Cloud Coverage.<br /><br />This endpoint shows which
        cloud regions are available for the `location_code` your Virtual Cross Connect
        will be provisioned in.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[cloud_provider], filter[cloud_provider_region], filter[location.region],
              filter[location.site], filter[location.pop], filter[location.code]

          filters:
              Consolidated filters parameter (deepObject style). Originally:
              filters[available_bandwidth][contains]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/virtual_cross_connects_coverage",
            page=SyncDefaultFlatPagination[VirtualCrossConnectsCoverageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "filters": filters,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    virtual_cross_connects_coverage_list_params.VirtualCrossConnectsCoverageListParams,
                ),
            ),
            model=VirtualCrossConnectsCoverageListResponse,
        )


class AsyncVirtualCrossConnectsCoverageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVirtualCrossConnectsCoverageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVirtualCrossConnectsCoverageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVirtualCrossConnectsCoverageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVirtualCrossConnectsCoverageResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: virtual_cross_connects_coverage_list_params.Filter | Omit = omit,
        filters: virtual_cross_connects_coverage_list_params.Filters | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        VirtualCrossConnectsCoverageListResponse, AsyncDefaultFlatPagination[VirtualCrossConnectsCoverageListResponse]
    ]:
        """
        List Virtual Cross Connects Cloud Coverage.<br /><br />This endpoint shows which
        cloud regions are available for the `location_code` your Virtual Cross Connect
        will be provisioned in.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[cloud_provider], filter[cloud_provider_region], filter[location.region],
              filter[location.site], filter[location.pop], filter[location.code]

          filters:
              Consolidated filters parameter (deepObject style). Originally:
              filters[available_bandwidth][contains]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/virtual_cross_connects_coverage",
            page=AsyncDefaultFlatPagination[VirtualCrossConnectsCoverageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "filters": filters,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    virtual_cross_connects_coverage_list_params.VirtualCrossConnectsCoverageListParams,
                ),
            ),
            model=VirtualCrossConnectsCoverageListResponse,
        )


class VirtualCrossConnectsCoverageResourceWithRawResponse:
    def __init__(self, virtual_cross_connects_coverage: VirtualCrossConnectsCoverageResource) -> None:
        self._virtual_cross_connects_coverage = virtual_cross_connects_coverage

        self.list = to_raw_response_wrapper(
            virtual_cross_connects_coverage.list,
        )


class AsyncVirtualCrossConnectsCoverageResourceWithRawResponse:
    def __init__(self, virtual_cross_connects_coverage: AsyncVirtualCrossConnectsCoverageResource) -> None:
        self._virtual_cross_connects_coverage = virtual_cross_connects_coverage

        self.list = async_to_raw_response_wrapper(
            virtual_cross_connects_coverage.list,
        )


class VirtualCrossConnectsCoverageResourceWithStreamingResponse:
    def __init__(self, virtual_cross_connects_coverage: VirtualCrossConnectsCoverageResource) -> None:
        self._virtual_cross_connects_coverage = virtual_cross_connects_coverage

        self.list = to_streamed_response_wrapper(
            virtual_cross_connects_coverage.list,
        )


class AsyncVirtualCrossConnectsCoverageResourceWithStreamingResponse:
    def __init__(self, virtual_cross_connects_coverage: AsyncVirtualCrossConnectsCoverageResource) -> None:
        self._virtual_cross_connects_coverage = virtual_cross_connects_coverage

        self.list = async_to_streamed_response_wrapper(
            virtual_cross_connects_coverage.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .filter import Filter as Filter
from .filter_param import FilterParam as FilterParam
from .usage_report_retrieve_speech_to_text_params import (
    UsageReportRetrieveSpeechToTextParams as UsageReportRetrieveSpeechToTextParams,
)
from .usage_report_retrieve_speech_to_text_response import (
    UsageReportRetrieveSpeechToTextResponse as UsageReportRetrieveSpeechToTextResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CallInitiateParams"]


class CallInitiateParams(TypedDict, total=False):
    from_: Required[Annotated[str, PropertyInfo(alias="From")]]
    """The phone number of the party that initiated the call.

    Phone numbers are formatted with a `+` and country code.
    """

    to: Required[Annotated[str, PropertyInfo(alias="To")]]
    """The phone number of the called party.

    Phone numbers are formatted with a `+` and country code.
    """

    async_amd: Annotated[bool, PropertyInfo(alias="AsyncAmd")]
    """Select whether to perform answering machine detection in the background.

    By default execution is blocked until Answering Machine Detection is completed.
    """

    async_amd_status_callback: Annotated[str, PropertyInfo(alias="AsyncAmdStatusCallback")]
    """URL destination for Telnyx to send AMD callback events to for the call."""

    async_amd_status_callback_method: Annotated[
        Literal["GET", "POST"], PropertyInfo(alias="AsyncAmdStatusCallbackMethod")
    ]
    """HTTP request type used for `AsyncAmdStatusCallback`.

    The default value is inherited from TeXML Application setting.
    """

    caller_id: Annotated[str, PropertyInfo(alias="CallerId")]
    """
    To be used as the caller id name (SIP From Display Name) presented to the
    destination (`To` number). The string should have a maximum of 128 characters,
    containing only letters, numbers, spaces, and `-_~!.+` special characters. If
    ommited, the display name will be the same as the number in the `From` field.
    """

    cancel_playback_on_detect_message_end: Annotated[bool, PropertyInfo(alias="CancelPlaybackOnDetectMessageEnd")]
    """Whether to cancel ongoing playback on `greeting ended` detection.

    Defaults to `true`.
    """

    cancel_playback_on_machine_detection: Annotated[bool, PropertyInfo(alias="CancelPlaybackOnMachineDetection")]
    """Whether to cancel ongoing playback on `machine` detection. Defaults to `true`."""

    detection_mode: Annotated[Literal["Premium", "Regular"], PropertyInfo(alias="DetectionMode")]
    """Allows you to chose between Premium and Standard detections."""

    fallback_url: Annotated[str, PropertyInfo(alias="FallbackUrl")]
    """
    A failover URL for which Telnyx will retrieve the TeXML call instructions if the
    `Url` is not responding.
    """

    machine_detection: Annotated[
        Literal["Enable", "Disable", "DetectMessageEnd"], PropertyInfo(alias="MachineDetection")
    ]
    """Enables Answering Machine Detection."""

    machine_detection_silence_timeout: Annotated[int, PropertyInfo(alias="MachineDetectionSilenceTimeout")]
    """If initial silence duration is greater than this value, consider it a machine.

    Ignored when `premium` detection is used.
    """

    machine_detection_speech_end_threshold: Annotated[int, PropertyInfo(alias="MachineDetectionSpeechEndThreshold")]
    """
    Silence duration threshold after a greeting message or voice for it be
    considered human. Ignored when `premium` detection is used.
    """

    machine_detection_speech_threshold: Annotated[int, PropertyInfo(alias="MachineDetectionSpeechThreshold")]
    """Maximum threshold of a human greeting.

    If greeting longer than this value, considered machine. Ignored when `premium`
    detection is used.
    """

    machine_detection_timeout: Annotated[int, PropertyInfo(alias="MachineDetectionTimeout")]
    """Maximum timeout threshold in milliseconds for overall detection."""

    preferred_codecs: Annotated[str, PropertyInfo(alias="PreferredCodecs")]
    """The list of comma-separated codecs to be offered on a call."""

    record: Annotated[bool, PropertyInfo(alias="Record")]
    """Whether to record the entire participant's call leg. Defaults to `false`."""

    recording_channels: Annotated[Literal["mono", "dual"], PropertyInfo(alias="RecordingChannels")]
    """The number of channels in the final recording. Defaults to `mono`."""

    recording_status_callback: Annotated[str, PropertyInfo(alias="RecordingStatusCallback")]
    """The URL the recording callbacks will be sent to."""

    recording_status_callback_event: Annotated[str, PropertyInfo(alias="RecordingStatusCallbackEvent")]
    """
    The changes to the recording's state that should generate a call to
    `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
    Separate multiple values with a space. Defaults to `completed`.
    """

    recording_status_callback_method: Annotated[
        Literal["GET", "POST"], PropertyInfo(alias="RecordingStatusCallbackMethod")
    ]
    """HTTP request type used for `RecordingStatusCallback`. Defaults to `POST`."""

    recording_timeout: Annotated[int, PropertyInfo(alias="RecordingTimeout")]
    """
    The number of seconds that Telnyx will wait for the recording to be stopped if
    silence is detected. The timer only starts when the speech is detected. Please
    note that the transcription is used to detect silence and the related charge
    will be applied. The minimum value is 0. The default value is 0 (infinite)
    """

    recording_track: Annotated[Literal["inbound", "outbound", "both"], PropertyInfo(alias="RecordingTrack")]
    """The audio track to record for the call. The default is `both`."""

    sip_auth_password: Annotated[str, PropertyInfo(alias="SipAuthPassword")]
    """The password to use for SIP authentication."""

    sip_auth_username: Annotated[str, PropertyInfo(alias="SipAuthUsername")]
    """The username to use for SIP authentication."""

    status_callback: Annotated[str, PropertyInfo(alias="StatusCallback")]
    """URL destination for Telnyx to send status callback events to for the call."""

    status_callback_event: Annotated[
        Literal["initiated", "ringing", "answered", "completed"], PropertyInfo(alias="StatusCallbackEvent")
    ]
    """The call events for which Telnyx should send a webhook.

    Multiple events can be defined when separated by a space.
    """

    status_callback_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="StatusCallbackMethod")]
    """HTTP request type used for `StatusCallback`."""

    trim: Annotated[Literal["trim-silence", "do-not-trim"], PropertyInfo(alias="Trim")]
    """Whether to trim any leading and trailing silence from the recording.

    Defaults to `trim-silence`.
    """

    url: Annotated[str, PropertyInfo(alias="Url")]
    """The URL from which Telnyx will retrieve the TeXML call instructions."""

    url_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="UrlMethod")]
    """HTTP request type used for `Url`.

    The default value is inherited from TeXML Application setting.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pwg_assigned_resources_summary import PwgAssignedResourcesSummary
from .private_wireless_gateway_status import PrivateWirelessGatewayStatus

__all__ = ["PrivateWirelessGateway"]


class PrivateWirelessGateway(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    assigned_resources: Optional[List[PwgAssignedResourcesSummary]] = None
    """
    A list of the resources that have been assigned to the Private Wireless Gateway.
    """

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    ip_range: Optional[str] = None
    """IP block used to assign IPs to the SIM cards in the Private Wireless Gateway."""

    name: Optional[str] = None
    """The private wireless gateway name."""

    network_id: Optional[str] = None
    """The identification of the related network resource."""

    record_type: Optional[str] = None

    region_code: Optional[str] = None
    """The name of the region where the Private Wireless Gateway is deployed."""

    status: Optional[PrivateWirelessGatewayStatus] = None
    """The current status or failure details of the Private Wireless Gateway."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .number_pool_settings import NumberPoolSettings
from .url_shortener_settings import URLShortenerSettings

__all__ = ["MessagingProfile"]


class MessagingProfile(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    alpha_sender: Optional[str] = None
    """
    The alphanumeric sender ID to use when sending to destinations that require an
    alphanumeric sender ID.
    """

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    daily_spend_limit: Optional[str] = None
    """
    The maximum amount of money (in USD) that can be spent by this profile before
    midnight UTC.
    """

    daily_spend_limit_enabled: Optional[bool] = None
    """Whether to enforce the value configured by `daily_spend_limit`."""

    enabled: Optional[bool] = None
    """Specifies whether the messaging profile is enabled or not."""

    health_webhook_url: Optional[str] = None
    """DEPRECATED: health check url service checking"""

    mms_fall_back_to_sms: Optional[bool] = None
    """enables SMS fallback for MMS messages."""

    mms_transcoding: Optional[bool] = None
    """enables automated resizing of MMS media."""

    mobile_only: Optional[bool] = None
    """Send messages only to mobile phone numbers."""

    name: Optional[str] = None
    """A user friendly name for the messaging profile."""

    number_pool_settings: Optional[NumberPoolSettings] = None
    """
    Number Pool allows you to send messages from a pool of numbers of different
    types, assigning weights to each type. The pool consists of all the long code
    and toll free numbers assigned to the messaging profile.

    To disable this feature, set the object field to `null`.
    """

    record_type: Optional[Literal["messaging_profile"]] = None
    """Identifies the type of the resource."""

    redaction_enabled: Optional[bool] = None
    """Indicates whether message content redaction is enabled for this profile."""

    redaction_level: Optional[int] = None
    """
    Determines how much information is redacted in messages for privacy or
    compliance purposes.
    """

    smart_encoding: Optional[bool] = None
    """Enables automatic character encoding optimization for SMS messages.

    When enabled, the system automatically selects the most efficient encoding
    (GSM-7 or UCS-2) based on message content to maximize character limits and
    minimize costs.
    """

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""

    url_shortener_settings: Optional[URLShortenerSettings] = None
    """
    The URL shortener feature allows automatic replacement of URLs that were
    generated using a public URL shortener service. Some examples include bit.do,
    bit.ly, goo.gl, ht.ly, is.gd, ow.ly, rebrand.ly, t.co, tiny.cc, and tinyurl.com.
    Such URLs are replaced with with links generated by Telnyx. The use of custom
    links can improve branding and message deliverability.

    To disable this feature, set the object field to `null`.
    """

    v1_secret: Optional[str] = None
    """Secret used to authenticate with v1 endpoints."""

    webhook_api_version: Optional[Literal["1", "2", "2010-04-01"]] = None
    """
    Determines which webhook format will be used, Telnyx API v1, v2, or a legacy
    2010-04-01 format.
    """

    webhook_failover_url: Optional[str] = None
    """
    The failover URL where webhooks related to this messaging profile will be sent
    if sending to the primary URL fails.
    """

    webhook_url: Optional[str] = None
    """The URL where webhooks related to this messaging profile will be sent."""

    whitelisted_destinations: Optional[List[str]] = None
    """Destinations to which the messaging profile is allowed to send.

    The elements in the list must be valid ISO 3166-1 alpha-2 country codes. If set
    to `["*"]`, all destinations will be allowed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr
from .dtmf_type import DtmfType
from .encrypted_media import EncryptedMedia
from .inbound_ip_param import InboundIPParam
from .outbound_ip_param import OutboundIPParam
from .anchorsite_override import AnchorsiteOverride
from .connection_rtcp_settings_param import ConnectionRtcpSettingsParam

__all__ = ["IPConnectionUpdateParams", "NoiseSuppressionDetails"]


class IPConnectionUpdateParams(TypedDict, total=False):
    active: bool
    """Defaults to true"""

    anchorsite_override: AnchorsiteOverride
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    android_push_credential_id: Optional[str]
    """The uuid of the push credential for Android"""

    call_cost_in_webhooks: bool
    """Specifies if call cost webhooks should be sent for this connection."""

    connection_name: str

    default_on_hold_comfort_noise_enabled: bool
    """When enabled, Telnyx will generate comfort noise when you place the call on
    hold.

    If disabled, you will need to generate comfort noise or on hold music to avoid
    RTP timeout.
    """

    dtmf_type: DtmfType
    """Sets the type of DTMF digits sent from Telnyx to this Connection.

    Note that DTMF digits sent to Telnyx will be accepted in all formats.
    """

    encode_contact_header_enabled: bool
    """
    Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
    scenarios.
    """

    encrypted_media: Optional[EncryptedMedia]
    """Enable use of SRTP for encryption.

    Cannot be set if the transport_portocol is TLS.
    """

    inbound: InboundIPParam

    ios_push_credential_id: Optional[str]
    """The uuid of the push credential for Ios"""

    noise_suppression: Literal["inbound", "outbound", "both", "disabled"]
    """Controls when noise suppression is applied to calls.

    When set to 'inbound', noise suppression is applied to incoming audio. When set
    to 'outbound', it's applied to outgoing audio. When set to 'both', it's applied
    in both directions. When set to 'disabled', noise suppression is turned off.
    """

    noise_suppression_details: NoiseSuppressionDetails
    """Configuration options for noise suppression.

    These settings are stored regardless of the noise_suppression value, but only
    take effect when noise_suppression is not 'disabled'. If you disable noise
    suppression and later re-enable it, the previously configured settings will be
    used.
    """

    onnet_t38_passthrough_enabled: bool
    """
    Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
    if both are on the Telnyx network. If this is disabled, Telnyx will be able to
    use T38 on just one leg of the call depending on each leg's settings.
    """

    outbound: OutboundIPParam

    rtcp_settings: ConnectionRtcpSettingsParam

    tags: SequenceNotStr[str]
    """Tags associated with the connection."""

    transport_protocol: Literal["UDP", "TCP", "TLS"]
    """One of UDP, TLS, or TCP.

    Applies only to connections with IP authentication or FQDN authentication.
    """

    webhook_api_version: Literal["1", "2"]
    """Determines which webhook format will be used, Telnyx API v1 or v2."""

    webhook_event_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: str
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int]
    """Specifies how many seconds to wait before timing out a webhook."""


class NoiseSuppressionDetails(TypedDict, total=False):
    """Configuration options for noise suppression.

    These settings are stored regardless of the noise_suppression value, but only take effect when noise_suppression is not 'disabled'. If you disable noise suppression and later re-enable it, the previously configured settings will be used.
    """

    attenuation_limit: int
    """The attenuation limit value for the selected engine.

    Default values vary by engine: 0 for 'denoiser', 80 for 'deep_filter_net',
    'deep_filter_net_large', and all Krisp engines ('krisp_viva_tel',
    'krisp_viva_tel_lite', 'krisp_viva_promodel', 'krisp_viva_ss').
    """

    engine: Literal[
        "denoiser",
        "deep_filter_net",
        "deep_filter_net_large",
        "krisp_viva_tel",
        "krisp_viva_tel_lite",
        "krisp_viva_promodel",
        "krisp_viva_ss",
    ]
    """The noise suppression engine to use.

    'denoiser' is the default engine. 'deep_filter_net' and 'deep_filter_net_large'
    are alternative engines with different performance characteristics. Krisp
    engines ('krisp_viva_tel', 'krisp_viva_tel_lite', 'krisp_viva_promodel',
    'krisp_viva_ss') provide advanced noise suppression capabilities.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ActionStartForkingParams"]


class ActionStartForkingParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    rx: str
    """
    The network target, <udp:ip_address:port>, where the call's incoming RTP media
    packets should be forwarded.
    """

    stream_type: Literal["decrypted"]
    """Optionally specify a media type to stream.

    If `decrypted` selected, Telnyx will decrypt incoming SIP media before forking
    to the target. `rx` and `tx` are required fields if `decrypted` selected.
    """

    tx: str
    """
    The network target, <udp:ip_address:port>, where the call's outgoing RTP media
    packets should be forwarded.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["InferenceEmbeddingBucketIDs"]


class InferenceEmbeddingBucketIDs(BaseModel):
    bucket_ids: List[str]
    """
    List of
    [embedded storage buckets](https://developers.telnyx.com/api-reference/embeddings/embed-documents)
    to use for retrieval-augmented generation.
    """

    max_num_results: Optional[int] = None
    """The maximum number of results to retrieve as context for the language model."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import ip_list_params, ip_create_params, ip_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from ..types.ip import IP
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.ip_create_response import IPCreateResponse
from ..types.ip_delete_response import IPDeleteResponse
from ..types.ip_update_response import IPUpdateResponse
from ..types.ip_retrieve_response import IPRetrieveResponse

__all__ = ["IPsResource", "AsyncIPsResource"]


class IPsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return IPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return IPsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        ip_address: str,
        connection_id: str | Omit = omit,
        port: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPCreateResponse:
        """
        Create a new IP object.

        Args:
          ip_address: IP adddress represented by this resource.

          connection_id: ID of the IP Connection to which this IP should be attached.

          port: Port to use when connecting to this IP.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ips",
            body=maybe_transform(
                {
                    "ip_address": ip_address,
                    "connection_id": connection_id,
                    "port": port,
                },
                ip_create_params.IPCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPRetrieveResponse:
        """
        Return the details regarding a specific IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        ip_address: str,
        connection_id: str | Omit = omit,
        port: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPUpdateResponse:
        """
        Update the details of a specific IP.

        Args:
          ip_address: IP adddress represented by this resource.

          connection_id: ID of the IP Connection to which this IP should be attached.

          port: Port to use when connecting to this IP.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/ips/{id}",
            body=maybe_transform(
                {
                    "ip_address": ip_address,
                    "connection_id": connection_id,
                    "port": port,
                },
                ip_update_params.IPUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPUpdateResponse,
        )

    def list(
        self,
        *,
        filter: ip_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[IP]:
        """
        Get all IPs belonging to the user that match the given filters.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_id], filter[ip_address], filter[port]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ips",
            page=SyncDefaultFlatPagination[IP],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    ip_list_params.IPListParams,
                ),
            ),
            model=IP,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPDeleteResponse:
        """
        Delete an IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPDeleteResponse,
        )


class AsyncIPsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncIPsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        ip_address: str,
        connection_id: str | Omit = omit,
        port: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPCreateResponse:
        """
        Create a new IP object.

        Args:
          ip_address: IP adddress represented by this resource.

          connection_id: ID of the IP Connection to which this IP should be attached.

          port: Port to use when connecting to this IP.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ips",
            body=await async_maybe_transform(
                {
                    "ip_address": ip_address,
                    "connection_id": connection_id,
                    "port": port,
                },
                ip_create_params.IPCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPRetrieveResponse:
        """
        Return the details regarding a specific IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        ip_address: str,
        connection_id: str | Omit = omit,
        port: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPUpdateResponse:
        """
        Update the details of a specific IP.

        Args:
          ip_address: IP adddress represented by this resource.

          connection_id: ID of the IP Connection to which this IP should be attached.

          port: Port to use when connecting to this IP.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/ips/{id}",
            body=await async_maybe_transform(
                {
                    "ip_address": ip_address,
                    "connection_id": connection_id,
                    "port": port,
                },
                ip_update_params.IPUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPUpdateResponse,
        )

    def list(
        self,
        *,
        filter: ip_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[IP, AsyncDefaultFlatPagination[IP]]:
        """
        Get all IPs belonging to the user that match the given filters.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_id], filter[ip_address], filter[port]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ips",
            page=AsyncDefaultFlatPagination[IP],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    ip_list_params.IPListParams,
                ),
            ),
            model=IP,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPDeleteResponse:
        """
        Delete an IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPDeleteResponse,
        )


class IPsResourceWithRawResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.create = to_raw_response_wrapper(
            ips.create,
        )
        self.retrieve = to_raw_response_wrapper(
            ips.retrieve,
        )
        self.update = to_raw_response_wrapper(
            ips.update,
        )
        self.list = to_raw_response_wrapper(
            ips.list,
        )
        self.delete = to_raw_response_wrapper(
            ips.delete,
        )


class AsyncIPsResourceWithRawResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.create = async_to_raw_response_wrapper(
            ips.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            ips.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            ips.update,
        )
        self.list = async_to_raw_response_wrapper(
            ips.list,
        )
        self.delete = async_to_raw_response_wrapper(
            ips.delete,
        )


class IPsResourceWithStreamingResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.create = to_streamed_response_wrapper(
            ips.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ips.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            ips.update,
        )
        self.list = to_streamed_response_wrapper(
            ips.list,
        )
        self.delete = to_streamed_response_wrapper(
            ips.delete,
        )


class AsyncIPsResourceWithStreamingResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.create = async_to_streamed_response_wrapper(
            ips.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ips.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            ips.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ips.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ips.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PrivacySettingsParam"]


class PrivacySettingsParam(TypedDict, total=False):
    data_retention: bool
    """If true, conversation history and insights will be stored.

    If false, they will not be stored. This in‑tool toggle governs solely the
    retention of conversation history and insights via the AI assistant. It has no
    effect on any separate recording, transcription, or storage configuration that
    you have set at the account, number, or application level. All such external
    settings remain in force regardless of your selection here.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import (
    mobile_voice_connection_list_params,
    mobile_voice_connection_create_params,
    mobile_voice_connection_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.mobile_voice_connection import MobileVoiceConnection
from ..types.mobile_voice_connection_create_response import MobileVoiceConnectionCreateResponse
from ..types.mobile_voice_connection_delete_response import MobileVoiceConnectionDeleteResponse
from ..types.mobile_voice_connection_update_response import MobileVoiceConnectionUpdateResponse
from ..types.mobile_voice_connection_retrieve_response import MobileVoiceConnectionRetrieveResponse

__all__ = ["MobileVoiceConnectionsResource", "AsyncMobileVoiceConnectionsResource"]


class MobileVoiceConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MobileVoiceConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MobileVoiceConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MobileVoiceConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MobileVoiceConnectionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        active: bool | Omit = omit,
        connection_name: str | Omit = omit,
        inbound: mobile_voice_connection_create_params.Inbound | Omit = omit,
        outbound: mobile_voice_connection_create_params.Outbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionCreateResponse:
        """
        Create a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/mobile_voice_connections",
            body=maybe_transform(
                {
                    "active": active,
                    "connection_name": connection_name,
                    "inbound": inbound,
                    "outbound": outbound,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                mobile_voice_connection_create_params.MobileVoiceConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionRetrieveResponse:
        """
        Retrieve a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v2/mobile_voice_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        connection_name: str | Omit = omit,
        inbound: mobile_voice_connection_update_params.Inbound | Omit = omit,
        outbound: mobile_voice_connection_update_params.Outbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionUpdateResponse:
        """
        Update a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/v2/mobile_voice_connections/{id}",
            body=maybe_transform(
                {
                    "active": active,
                    "connection_name": connection_name,
                    "inbound": inbound,
                    "outbound": outbound,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                mobile_voice_connection_update_params.MobileVoiceConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter_connection_name_contains: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MobileVoiceConnection]:
        """
        List Mobile Voice Connections

        Args:
          filter_connection_name_contains: Filter by connection name containing the given string

          page_number: The page number to load

          page_size: The size of the page

          sort: Sort by field (e.g., created_at, connection_name, active). Prefix with - for
              descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/mobile_voice_connections",
            page=SyncDefaultFlatPagination[MobileVoiceConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_connection_name_contains": filter_connection_name_contains,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    mobile_voice_connection_list_params.MobileVoiceConnectionListParams,
                ),
            ),
            model=MobileVoiceConnection,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionDeleteResponse:
        """
        Delete a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/v2/mobile_voice_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionDeleteResponse,
        )


class AsyncMobileVoiceConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMobileVoiceConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMobileVoiceConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMobileVoiceConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMobileVoiceConnectionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        active: bool | Omit = omit,
        connection_name: str | Omit = omit,
        inbound: mobile_voice_connection_create_params.Inbound | Omit = omit,
        outbound: mobile_voice_connection_create_params.Outbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionCreateResponse:
        """
        Create a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/mobile_voice_connections",
            body=await async_maybe_transform(
                {
                    "active": active,
                    "connection_name": connection_name,
                    "inbound": inbound,
                    "outbound": outbound,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                mobile_voice_connection_create_params.MobileVoiceConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionRetrieveResponse:
        """
        Retrieve a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v2/mobile_voice_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        connection_name: str | Omit = omit,
        inbound: mobile_voice_connection_update_params.Inbound | Omit = omit,
        outbound: mobile_voice_connection_update_params.Outbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: Optional[str] | Omit = omit,
        webhook_timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionUpdateResponse:
        """
        Update a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/v2/mobile_voice_connections/{id}",
            body=await async_maybe_transform(
                {
                    "active": active,
                    "connection_name": connection_name,
                    "inbound": inbound,
                    "outbound": outbound,
                    "tags": tags,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                mobile_voice_connection_update_params.MobileVoiceConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter_connection_name_contains: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MobileVoiceConnection, AsyncDefaultFlatPagination[MobileVoiceConnection]]:
        """
        List Mobile Voice Connections

        Args:
          filter_connection_name_contains: Filter by connection name containing the given string

          page_number: The page number to load

          page_size: The size of the page

          sort: Sort by field (e.g., created_at, connection_name, active). Prefix with - for
              descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/mobile_voice_connections",
            page=AsyncDefaultFlatPagination[MobileVoiceConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_connection_name_contains": filter_connection_name_contains,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    mobile_voice_connection_list_params.MobileVoiceConnectionListParams,
                ),
            ),
            model=MobileVoiceConnection,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MobileVoiceConnectionDeleteResponse:
        """
        Delete a Mobile Voice Connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/v2/mobile_voice_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MobileVoiceConnectionDeleteResponse,
        )


class MobileVoiceConnectionsResourceWithRawResponse:
    def __init__(self, mobile_voice_connections: MobileVoiceConnectionsResource) -> None:
        self._mobile_voice_connections = mobile_voice_connections

        self.create = to_raw_response_wrapper(
            mobile_voice_connections.create,
        )
        self.retrieve = to_raw_response_wrapper(
            mobile_voice_connections.retrieve,
        )
        self.update = to_raw_response_wrapper(
            mobile_voice_connections.update,
        )
        self.list = to_raw_response_wrapper(
            mobile_voice_connections.list,
        )
        self.delete = to_raw_response_wrapper(
            mobile_voice_connections.delete,
        )


class AsyncMobileVoiceConnectionsResourceWithRawResponse:
    def __init__(self, mobile_voice_connections: AsyncMobileVoiceConnectionsResource) -> None:
        self._mobile_voice_connections = mobile_voice_connections

        self.create = async_to_raw_response_wrapper(
            mobile_voice_connections.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            mobile_voice_connections.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            mobile_voice_connections.update,
        )
        self.list = async_to_raw_response_wrapper(
            mobile_voice_connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            mobile_voice_connections.delete,
        )


class MobileVoiceConnectionsResourceWithStreamingResponse:
    def __init__(self, mobile_voice_connections: MobileVoiceConnectionsResource) -> None:
        self._mobile_voice_connections = mobile_voice_connections

        self.create = to_streamed_response_wrapper(
            mobile_voice_connections.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            mobile_voice_connections.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            mobile_voice_connections.update,
        )
        self.list = to_streamed_response_wrapper(
            mobile_voice_connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            mobile_voice_connections.delete,
        )


class AsyncMobileVoiceConnectionsResourceWithStreamingResponse:
    def __init__(self, mobile_voice_connections: AsyncMobileVoiceConnectionsResource) -> None:
        self._mobile_voice_connections = mobile_voice_connections

        self.create = async_to_streamed_response_wrapper(
            mobile_voice_connections.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            mobile_voice_connections.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            mobile_voice_connections.update,
        )
        self.list = async_to_streamed_response_wrapper(
            mobile_voice_connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            mobile_voice_connections.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["UploadCreateResponse"]


class UploadCreateResponse(BaseModel):
    success: Optional[bool] = None
    """Describes wether or not the operation was successful"""

    ticket_id: Optional[str] = None
    """Ticket id of the upload request"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["PaginationResponse"]


class PaginationResponse(BaseModel):
    page_number: int
    """The current page number."""

    page_size: int
    """The number of results per page."""

    total_pages: int
    """Total number of pages from the results."""

    total_results: int
    """Total number of results returned."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["UserGetGroupsReportResponse", "Data", "DataGroup"]


class DataGroup(BaseModel):
    """A reference to a group that a user belongs to."""

    id: str
    """The unique identifier of the group."""

    name: str
    """The name of the group."""


class Data(BaseModel):
    """An organization user with their group memberships always included."""

    id: str
    """Identifies the specific resource."""

    created_at: str
    """ISO 8601 formatted date indicating when the resource was created."""

    email: str
    """The email address of the user."""

    groups: List[DataGroup]
    """The groups the user belongs to."""

    record_type: str
    """Identifies the type of the resource.

    Can be 'organization_owner' or 'organization_sub_user'.
    """

    user_status: Literal["enabled", "disabled", "blocked"]
    """The status of the account."""

    last_sign_in_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource last signed into the
    portal.

    Null if the user has never signed in.
    """

    organization_user_bypasses_sso: Optional[bool] = None
    """
    Indicates whether this user is allowed to bypass SSO and use password
    authentication.
    """


class UserGetGroupsReportResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["SimCardListParams", "Filter"]


class SimCardListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for SIM cards (deepObject style).

    Originally: filter[tags], filter[iccid], filter[status]
    """

    filter_sim_card_group_id: Annotated[str, PropertyInfo(alias="filter[sim_card_group_id]")]
    """A valid SIM card group ID."""

    include_sim_card_group: bool
    """It includes the associated SIM card group object in the response when present."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Literal["current_billing_period_consumed_data.amount", "-current_billing_period_consumed_data.amount"]
    """Sorts SIM cards by the given field.

    Defaults to ascending order unless field is prefixed with a minus sign.
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter for SIM cards (deepObject style).

    Originally: filter[tags], filter[iccid], filter[status]
    """

    iccid: str
    """A search string to partially match for the SIM card's ICCID."""

    status: List[Literal["enabled", "disabled", "standby", "data_limit_exceeded", "unauthorized_imei"]]
    """Filter by a SIM card's status."""

    tags: SequenceNotStr[str]
    """
    A list of SIM card tags to filter on.<br/><br/> If the SIM card contains
    <b><i>all</i></b> of the given <code>tags</code> they will be found.<br/><br/>
    For example, if the SIM cards have the following tags: <ul>

      <li><code>['customers', 'staff', 'test']</code>
      <li><code>['test']</code></li>
      <li><code>['customers']</code></li>
    </ul>
    Searching for <code>['customers', 'test']</code> returns only the first because it's the only one with both tags.<br/> Searching for <code>test</code> returns the first two SIMs, because both of them have such tag.<br/> Searching for <code>customers</code> returns the first and last SIMs.<br/>
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .inference_embedding import InferenceEmbedding

__all__ = ["AssistantsList"]


class AssistantsList(BaseModel):
    data: List[InferenceEmbedding]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["QueueRetrieveResponse", "Data"]


class Data(BaseModel):
    id: str
    """Uniquely identifies the queue"""

    average_wait_time_secs: int
    """
    The average time that the calls currently in the queue have spent waiting, given
    in seconds.
    """

    created_at: str
    """ISO 8601 formatted date of when the queue was created"""

    current_size: int
    """The number of calls currently in the queue"""

    max_size: int
    """The maximum number of calls allowed in the queue"""

    name: str
    """Name of the queue"""

    record_type: Literal["queue"]

    updated_at: str
    """ISO 8601 formatted date of when the queue was last updated"""


class QueueRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = [
    "PhoneNumbersRegulatoryRequirementRetrieveResponse",
    "Data",
    "DataRegionInformation",
    "DataRegulatoryRequirement",
    "DataRegulatoryRequirementAcceptanceCriteria",
]


class DataRegionInformation(BaseModel):
    region_name: Optional[str] = None

    region_type: Optional[str] = None


class DataRegulatoryRequirementAcceptanceCriteria(BaseModel):
    field_type: Optional[str] = None

    field_value: Optional[str] = None

    locality_limit: Optional[str] = None


class DataRegulatoryRequirement(BaseModel):
    id: Optional[str] = None

    acceptance_criteria: Optional[DataRegulatoryRequirementAcceptanceCriteria] = None

    description: Optional[str] = None

    example: Optional[str] = None

    field_type: Optional[str] = None

    label: Optional[str] = None

    record_type: Optional[str] = None


class Data(BaseModel):
    phone_number: Optional[str] = None

    phone_number_type: Optional[str] = None

    record_type: Optional[str] = None

    region_information: Optional[List[DataRegionInformation]] = None

    regulatory_requirements: Optional[List[DataRegulatoryRequirement]] = None


class PhoneNumbersRegulatoryRequirementRetrieveResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, TypedDict

__all__ = ["AdditionalDocumentCreateParams", "AdditionalDocument"]


class AdditionalDocumentCreateParams(TypedDict, total=False):
    additional_documents: Iterable[AdditionalDocument]


class AdditionalDocument(TypedDict, total=False):
    document_id: str
    """The document identification"""

    document_type: Literal["loa", "invoice", "csr", "other"]
    """The type of document being created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TelephonySettings", "NoiseSuppressionConfig"]


class NoiseSuppressionConfig(BaseModel):
    """Configuration for noise suppression.

    Only applicable when noise_suppression is 'deepfilternet'.
    """

    attenuation_limit: Optional[int] = None
    """Attenuation limit for noise suppression. Range: 0-100."""

    mode: Optional[Literal["advanced"]] = None
    """Mode for noise suppression configuration."""


class TelephonySettings(BaseModel):
    default_texml_app_id: Optional[str] = None
    """Default Texml App used for voice calls with your assistant.

    This will be created automatically on assistant creation.
    """

    noise_suppression: Optional[Literal["deepfilternet", "disabled"]] = None
    """The noise suppression engine to use.

    Use 'disabled' to turn off noise suppression.
    """

    noise_suppression_config: Optional[NoiseSuppressionConfig] = None
    """Configuration for noise suppression.

    Only applicable when noise_suppression is 'deepfilternet'.
    """

    supports_unauthenticated_web_calls: Optional[bool] = None
    """
    When enabled, allows users to interact with your AI assistant directly from your
    website without requiring authentication. This is required for FE widgets that
    work with assistants that have telephony enabled.
    """

    time_limit_secs: Optional[int] = None
    """Maximum duration in seconds for the AI assistant to participate on the call.

    When this limit is reached the assistant will be stopped. This limit does not
    apply to portions of a call without an active assistant (for instance, a call
    transferred to a human representative).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from .participants import (
    ParticipantsResource,
    AsyncParticipantsResource,
    ParticipantsResourceWithRawResponse,
    AsyncParticipantsResourceWithRawResponse,
    ParticipantsResourceWithStreamingResponse,
    AsyncParticipantsResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.texml.accounts import conference_update_params, conference_retrieve_conferences_params
from .....types.texml.accounts.conference_update_response import ConferenceUpdateResponse
from .....types.texml.accounts.conference_retrieve_response import ConferenceRetrieveResponse
from .....types.texml.accounts.conference_retrieve_recordings_response import ConferenceRetrieveRecordingsResponse
from .....types.texml.accounts.conference_retrieve_conferences_response import ConferenceRetrieveConferencesResponse
from .....types.texml.accounts.conference_retrieve_recordings_json_response import (
    ConferenceRetrieveRecordingsJsonResponse,
)

__all__ = ["ConferencesResource", "AsyncConferencesResource"]


class ConferencesResource(SyncAPIResource):
    @cached_property
    def participants(self) -> ParticipantsResource:
        return ParticipantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConferencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ConferencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConferencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ConferencesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveResponse:
        """
        Returns a conference resource.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveResponse,
        )

    def update(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        announce_method: Literal["GET", "POST"] | Omit = omit,
        announce_url: str | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceUpdateResponse:
        """
        Updates a conference resource.

        Args:
          announce_method: The HTTP method used to call the `AnnounceUrl`. Defaults to `POST`.

          announce_url: The URL we should call to announce something into the conference. The URL may
              return an MP3 file, a WAV file, or a TwiML document that contains `<Play>`,
              `<Say>`, `<Pause>`, or `<Redirect>` verbs.

          status: The new status of the resource. Specifying `completed` will end the conference
              and hang up all participants.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return self._post(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}",
            body=maybe_transform(
                {
                    "announce_method": announce_method,
                    "announce_url": announce_url,
                    "status": status,
                },
                conference_update_params.ConferenceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceUpdateResponse,
        )

    def retrieve_conferences(
        self,
        account_sid: str,
        *,
        date_created: str | Omit = omit,
        date_updated: str | Omit = omit,
        friendly_name: str | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        status: Literal["init", "in-progress", "completed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveConferencesResponse:
        """
        Lists conference resources.

        Args:
          date_created: Filters conferences by the creation date. Expected format is YYYY-MM-DD. Also
              accepts inequality operators, e.g. DateCreated>=2023-05-22.

          date_updated: Filters conferences by the time they were last updated. Expected format is
              YYYY-MM-DD. Also accepts inequality operators, e.g. DateUpdated>=2023-05-22.

          friendly_name: Filters conferences by their friendly name.

          page: The number of the page to be displayed, zero-indexed, should be used in
              conjuction with PageToken.

          page_size: The number of records to be displayed on a page

          page_token: Used to request the next page of results.

          status: Filters conferences by status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Conferences",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created": date_created,
                        "date_updated": date_updated,
                        "friendly_name": friendly_name,
                        "page": page,
                        "page_size": page_size,
                        "page_token": page_token,
                        "status": status,
                    },
                    conference_retrieve_conferences_params.ConferenceRetrieveConferencesParams,
                ),
            ),
            cast_to=ConferenceRetrieveConferencesResponse,
        )

    def retrieve_recordings(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveRecordingsResponse:
        """
        Lists conference recordings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Recordings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveRecordingsResponse,
        )

    def retrieve_recordings_json(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveRecordingsJsonResponse:
        """
        Returns recordings for a conference identified by conference_sid.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Recordings.json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveRecordingsJsonResponse,
        )


class AsyncConferencesResource(AsyncAPIResource):
    @cached_property
    def participants(self) -> AsyncParticipantsResource:
        return AsyncParticipantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConferencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConferencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConferencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncConferencesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveResponse:
        """
        Returns a conference resource.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveResponse,
        )

    async def update(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        announce_method: Literal["GET", "POST"] | Omit = omit,
        announce_url: str | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceUpdateResponse:
        """
        Updates a conference resource.

        Args:
          announce_method: The HTTP method used to call the `AnnounceUrl`. Defaults to `POST`.

          announce_url: The URL we should call to announce something into the conference. The URL may
              return an MP3 file, a WAV file, or a TwiML document that contains `<Play>`,
              `<Say>`, `<Pause>`, or `<Redirect>` verbs.

          status: The new status of the resource. Specifying `completed` will end the conference
              and hang up all participants.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return await self._post(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}",
            body=await async_maybe_transform(
                {
                    "announce_method": announce_method,
                    "announce_url": announce_url,
                    "status": status,
                },
                conference_update_params.ConferenceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceUpdateResponse,
        )

    async def retrieve_conferences(
        self,
        account_sid: str,
        *,
        date_created: str | Omit = omit,
        date_updated: str | Omit = omit,
        friendly_name: str | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        status: Literal["init", "in-progress", "completed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveConferencesResponse:
        """
        Lists conference resources.

        Args:
          date_created: Filters conferences by the creation date. Expected format is YYYY-MM-DD. Also
              accepts inequality operators, e.g. DateCreated>=2023-05-22.

          date_updated: Filters conferences by the time they were last updated. Expected format is
              YYYY-MM-DD. Also accepts inequality operators, e.g. DateUpdated>=2023-05-22.

          friendly_name: Filters conferences by their friendly name.

          page: The number of the page to be displayed, zero-indexed, should be used in
              conjuction with PageToken.

          page_size: The number of records to be displayed on a page

          page_token: Used to request the next page of results.

          status: Filters conferences by status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Conferences",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_created": date_created,
                        "date_updated": date_updated,
                        "friendly_name": friendly_name,
                        "page": page,
                        "page_size": page_size,
                        "page_token": page_token,
                        "status": status,
                    },
                    conference_retrieve_conferences_params.ConferenceRetrieveConferencesParams,
                ),
            ),
            cast_to=ConferenceRetrieveConferencesResponse,
        )

    async def retrieve_recordings(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveRecordingsResponse:
        """
        Lists conference recordings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Recordings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveRecordingsResponse,
        )

    async def retrieve_recordings_json(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConferenceRetrieveRecordingsJsonResponse:
        """
        Returns recordings for a conference identified by conference_sid.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Recordings.json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConferenceRetrieveRecordingsJsonResponse,
        )


class ConferencesResourceWithRawResponse:
    def __init__(self, conferences: ConferencesResource) -> None:
        self._conferences = conferences

        self.retrieve = to_raw_response_wrapper(
            conferences.retrieve,
        )
        self.update = to_raw_response_wrapper(
            conferences.update,
        )
        self.retrieve_conferences = to_raw_response_wrapper(
            conferences.retrieve_conferences,
        )
        self.retrieve_recordings = to_raw_response_wrapper(
            conferences.retrieve_recordings,
        )
        self.retrieve_recordings_json = to_raw_response_wrapper(
            conferences.retrieve_recordings_json,
        )

    @cached_property
    def participants(self) -> ParticipantsResourceWithRawResponse:
        return ParticipantsResourceWithRawResponse(self._conferences.participants)


class AsyncConferencesResourceWithRawResponse:
    def __init__(self, conferences: AsyncConferencesResource) -> None:
        self._conferences = conferences

        self.retrieve = async_to_raw_response_wrapper(
            conferences.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            conferences.update,
        )
        self.retrieve_conferences = async_to_raw_response_wrapper(
            conferences.retrieve_conferences,
        )
        self.retrieve_recordings = async_to_raw_response_wrapper(
            conferences.retrieve_recordings,
        )
        self.retrieve_recordings_json = async_to_raw_response_wrapper(
            conferences.retrieve_recordings_json,
        )

    @cached_property
    def participants(self) -> AsyncParticipantsResourceWithRawResponse:
        return AsyncParticipantsResourceWithRawResponse(self._conferences.participants)


class ConferencesResourceWithStreamingResponse:
    def __init__(self, conferences: ConferencesResource) -> None:
        self._conferences = conferences

        self.retrieve = to_streamed_response_wrapper(
            conferences.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            conferences.update,
        )
        self.retrieve_conferences = to_streamed_response_wrapper(
            conferences.retrieve_conferences,
        )
        self.retrieve_recordings = to_streamed_response_wrapper(
            conferences.retrieve_recordings,
        )
        self.retrieve_recordings_json = to_streamed_response_wrapper(
            conferences.retrieve_recordings_json,
        )

    @cached_property
    def participants(self) -> ParticipantsResourceWithStreamingResponse:
        return ParticipantsResourceWithStreamingResponse(self._conferences.participants)


class AsyncConferencesResourceWithStreamingResponse:
    def __init__(self, conferences: AsyncConferencesResource) -> None:
        self._conferences = conferences

        self.retrieve = async_to_streamed_response_wrapper(
            conferences.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            conferences.update,
        )
        self.retrieve_conferences = async_to_streamed_response_wrapper(
            conferences.retrieve_conferences,
        )
        self.retrieve_recordings = async_to_streamed_response_wrapper(
            conferences.retrieve_recordings,
        )
        self.retrieve_recordings_json = async_to_streamed_response_wrapper(
            conferences.retrieve_recordings_json,
        )

    @cached_property
    def participants(self) -> AsyncParticipantsResourceWithStreamingResponse:
        return AsyncParticipantsResourceWithStreamingResponse(self._conferences.participants)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SimCardGroupListParams"]


class SimCardGroupListParams(TypedDict, total=False):
    filter_name: Annotated[str, PropertyInfo(alias="filter[name]")]
    """A valid SIM card group name."""

    filter_private_wireless_gateway_id: Annotated[str, PropertyInfo(alias="filter[private_wireless_gateway_id]")]
    """A Private Wireless Gateway ID associated with the group."""

    filter_wireless_blocklist_id: Annotated[str, PropertyInfo(alias="filter[wireless_blocklist_id]")]
    """A Wireless Blocklist ID associated with the group."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """The page number to load."""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["FilterParam"]


class FilterParam(TypedDict, total=False):
    """Query filter criteria.

    Note: The first filter object must specify filter_type as 'and'. You cannot follow an 'or' with another 'and'.
    """

    billing_group: str
    """Billing group UUID to filter by"""

    cld: str
    """Called line identification (destination number)"""

    cld_filter: Literal["contains", "starts_with", "ends_with"]
    """Filter type for CLD matching"""

    cli: str
    """Calling line identification (caller ID)"""

    cli_filter: Literal["contains", "starts_with", "ends_with"]
    """Filter type for CLI matching"""

    filter_type: Literal["and", "or"]
    """Logical operator for combining filters"""

    tags_list: str
    """Tag name to filter by"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .addresses import (
    AddressesResource,
    AsyncAddressesResource,
    AddressesResourceWithRawResponse,
    AsyncAddressesResourceWithRawResponse,
    AddressesResourceWithStreamingResponse,
    AsyncAddressesResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "AddressesResource",
    "AsyncAddressesResource",
    "AddressesResourceWithRawResponse",
    "AsyncAddressesResourceWithRawResponse",
    "AddressesResourceWithStreamingResponse",
    "AsyncAddressesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ExternalConnectionUpdateLocationResponse", "Data"]


class Data(BaseModel):
    accepted_address_suggestions: Optional[bool] = None

    location_id: Optional[str] = None

    static_emergency_address_id: Optional[str] = None


class ExternalConnectionUpdateLocationResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["WirelessBlocklistCreateParams"]


class WirelessBlocklistCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name of the Wireless Blocklist."""

    type: Required[Literal["country", "mcc", "plmn"]]
    """The type of wireless blocklist."""

    values: Required[SequenceNotStr[str]]
    """Values to block. The values here depend on the `type` of Wireless Blocklist."""
